"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var assert = require("assert");

var _require = require("../core"),
    call = _require.call;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    cmds = _require2.cmds,
    functions = _require2.functions;

var _require3 = require("./test-util"),
    expectErrorEquality = _require3.expectErrorEquality;

var basic = functions.basic,
    usesThrowingInterpreter = functions.usesThrowingInterpreter,
    usesRejectingInterpreter = functions.usesRejectingInterpreter;
test("call should reject when a interpreter throws and is not caught",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee() {
  var expected;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return call({}, interpreters, usesThrowingInterpreter);

        case 3:
          _context.next = 13;
          break;

        case 5:
          _context.prev = 5;
          _context.t0 = _context["catch"](0);
          expected = new Error("oops");
          expected.fn = usesThrowingInterpreter;
          expected.index = 0;
          expected.step = 0;
          expected.command = cmds.die("oops");
          return _context.abrupt("return", expectErrorEquality(_context.t0, expected));

        case 13:
          fail("Function did not reject.");

        case 14:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this, [[0, 5]]);
})));
test("call should reject when a interpreter rejects and is not caught",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee2() {
  var expected;
  return regeneratorRuntime.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.prev = 0;
          _context2.next = 3;
          return call({}, interpreters, usesRejectingInterpreter);

        case 3:
          _context2.next = 13;
          break;

        case 5:
          _context2.prev = 5;
          _context2.t0 = _context2["catch"](0);
          expected = new Error("oops");
          expected.fn = usesRejectingInterpreter;
          expected.index = 0;
          expected.step = 0;
          expected.command = cmds.dieFromRejection("oops");
          return _context2.abrupt("return", expectErrorEquality(_context2.t0, expected));

        case 13:
          fail("Function did not reject.");

        case 14:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2, this, [[0, 5]]);
})));
test("call should reject if no context passed in",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee3() {
  return regeneratorRuntime.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.prev = 0;
          _context3.next = 3;
          return call(null, interpreters, basic);

        case 3:
          _context3.next = 9;
          break;

        case 5:
          _context3.prev = 5;
          _context3.t0 = _context3["catch"](0);
          assert.equal(_context3.t0.message, "context required");
          return _context3.abrupt("return");

        case 9:
          fail("did not throw");

        case 10:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee3, this, [[0, 5]]);
})));
test("call should reject if no function passed in",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee4() {
  return regeneratorRuntime.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.prev = 0;
          _context4.next = 3;
          return call({}, interpreters);

        case 3:
          _context4.next = 9;
          break;

        case 5:
          _context4.prev = 5;
          _context4.t0 = _context4["catch"](0);
          assert.equal(_context4.t0.message, "function required");
          return _context4.abrupt("return");

        case 9:
          fail("did not throw");

        case 10:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4, this, [[0, 5]]);
})));
//# sourceMappingURL=core.spec.js.map