"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var assert = require("assert");

var _require = require("../interpreters"),
    setImmediate = _require.setImmediate,
    setTimeout = _require.setTimeout,
    setInterval = _require.setInterval,
    globalVariable = _require.globalVariable;

var _require2 = require("../core"),
    call = _require2.call;

var _require3 = require("./common"),
    interpreters = _require3.interpreters,
    functions = _require3.functions,
    cmds = _require3.cmds;

var _require4 = require("./test-util"),
    sleep = _require4.sleep;

var coreInterpreters = require("../interpreters");

var coreCmds = require("../cmds");

test("setImmediate() should swallow errors to prevent node uncaught rejection errors",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee() {
  var context;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          context = {}; // Should not cause a node unhandled rejection error

          _context.next = 3;
          return setImmediate({
            cmd: cmds.die()
          }, {
            call: call,
            context: context,
            interpreters: interpreters
          });

        case 3:
          _context.next = 5;
          return sleep(10);

        case 5:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this);
})));
test("setImmediate() should reset stack, if resetStack is truthy",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee2() {
  var _marked, count, stack, onCommand, nestedCall, context, now;

  return regeneratorRuntime.wrap(function _callee2$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          nestedCall = function _ref3() {
            return regeneratorRuntime.wrap(function nestedCall$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    if (!(count >= 10)) {
                      _context2.next = 2;
                      break;
                    }

                    return _context2.abrupt("return");

                  case 2:
                    _context2.next = 4;
                    return coreCmds.setImmediate(coreCmds.call(nestedCall), true);

                  case 4:
                  case "end":
                    return _context2.stop();
                }
              }
            }, _marked, this);
          };

          _marked =
          /*#__PURE__*/
          regeneratorRuntime.mark(nestedCall);
          count = 0;

          onCommand = function onCommand(t) {
            if (t.command.type !== "call") return;
            count++;

            if (count >= 10) {
              stack = t.context.stack;
            }
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context3.next = 8;
          return call(context, coreInterpreters, nestedCall, "foo");

        case 8:
          _context3.next = 10;
          return sleep(50);

        case 10:
          expect(stack).toBeTruthy();
          expect(stack.length).toEqual(1);

        case 12:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee2, this);
})));
test("setTimeout() should swallow errors to prevent node uncaught rejection errors",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee3() {
  var context;
  return regeneratorRuntime.wrap(function _callee3$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          context = {}; // Should not cause a node unhandled rejection error

          _context4.next = 3;
          return setTimeout({
            cmd: cmds.die(),
            time: 0
          }, {
            call: call,
            context: context,
            interpreters: interpreters
          });

        case 3:
          _context4.next = 5;
          return sleep(10);

        case 5:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee3, this);
})));
test("setTimeout() should reset stack, if resetStack is truthy",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee4() {
  var _marked2, count, stack, onCommand, nestedCall, context, now;

  return regeneratorRuntime.wrap(function _callee4$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          nestedCall = function _ref6() {
            return regeneratorRuntime.wrap(function nestedCall$(_context5) {
              while (1) {
                switch (_context5.prev = _context5.next) {
                  case 0:
                    if (!(count >= 10)) {
                      _context5.next = 2;
                      break;
                    }

                    return _context5.abrupt("return");

                  case 2:
                    _context5.next = 4;
                    return coreCmds.setTimeout(coreCmds.call(nestedCall), 0, true);

                  case 4:
                  case "end":
                    return _context5.stop();
                }
              }
            }, _marked2, this);
          };

          _marked2 =
          /*#__PURE__*/
          regeneratorRuntime.mark(nestedCall);
          count = 0;

          onCommand = function onCommand(t) {
            if (t.command.type !== "call") return;
            count++;

            if (count >= 10) {
              stack = t.context.stack;
            }
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context6.next = 8;
          return call(context, coreInterpreters, nestedCall, "foo");

        case 8:
          _context6.next = 10;
          return sleep(50);

        case 10:
          expect(stack).toBeTruthy();
          expect(stack.length).toEqual(1);

        case 12:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee4, this);
})));
test("setInterval() should swallow errors to prevent node uncaught rejection errors",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee5() {
  var context, id;
  return regeneratorRuntime.wrap(function _callee5$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          context = {}; // Should not cause a node unhandled rejection error

          _context7.next = 3;
          return setInterval({
            cmd: cmds.die(),
            time: 0
          }, {
            call: call,
            context: context,
            interpreters: interpreters
          });

        case 3:
          id = _context7.sent;
          _context7.next = 6;
          return sleep(10);

        case 6:
          clearInterval(id);

        case 7:
        case "end":
          return _context7.stop();
      }
    }
  }, _callee5, this);
})));
test("setInterval() should reset stack, if resetStack is truthy",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee6() {
  var _marked3, count, stack, interval, onCommand, nestedCall, context;

  return regeneratorRuntime.wrap(function _callee6$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          nestedCall = function _ref9() {
            return regeneratorRuntime.wrap(function nestedCall$(_context8) {
              while (1) {
                switch (_context8.prev = _context8.next) {
                  case 0:
                    if (!(count >= 10)) {
                      _context8.next = 2;
                      break;
                    }

                    return _context8.abrupt("return");

                  case 2:
                    _context8.next = 4;
                    return coreCmds.setInterval(coreCmds.now(), 0, true);

                  case 4:
                    interval = _context8.sent;

                  case 5:
                  case "end":
                    return _context8.stop();
                }
              }
            }, _marked3, this);
          };

          _marked3 =
          /*#__PURE__*/
          regeneratorRuntime.mark(nestedCall);
          count = 0;

          onCommand = function onCommand(t) {
            if (t.command.type !== "now") return;
            count++;

            if (count >= 10) {
              stack = t.context.stack;
            }
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          _context9.next = 7;
          return call(context, coreInterpreters, nestedCall, "foo");

        case 7:
          _context9.next = 9;
          return sleep(50);

        case 9:
          clearInterval(interval);
          expect(stack).toBeTruthy();
          expect(stack.length).toEqual(1);

        case 12:
        case "end":
          return _context9.stop();
      }
    }
  }, _callee6, this);
})));
test("globalVariable() should use window if available", function () {
  global.window = {
    foo: "bar window"
  };
  var value = globalVariable(coreCmds.globalVariable("foo"));
  global.window = undefined;
  assert.equal(value, "bar window");
});
test("globalVariable() should global if window is not available ", function () {
  global.foo = "bar global";
  var value = globalVariable(coreCmds.globalVariable("foo"));
  assert.equal(value, "bar global");
});
//# sourceMappingURL=interpreters.spec.js.map