"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var _require = require("../core"),
    call = _require.call;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    cmds = _require2.cmds,
    functions = _require2.functions;

var basic = functions.basic,
    basicMultistep = functions.basicMultistep,
    basicParallel = functions.basicParallel,
    basicMultistepParallel = functions.basicMultistepParallel,
    basicEmpty = functions.basicEmpty,
    basicMultiArg = functions.basicMultiArg;
test("should be able to process a command",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee() {
  var expected, actual;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          expected = "foo";
          _context.next = 3;
          return call({}, interpreters, basic, expected);

        case 3:
          actual = _context.sent;
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this);
})));
test("should be able to accept array arguments",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee2() {
  var expected, actual;
  return regeneratorRuntime.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          expected = ["foo", "bar"];
          _context2.next = 3;
          return call({}, interpreters, basic, expected);

        case 3:
          actual = _context2.sent;
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2, this);
})));
test("should be able to handle multiple arguments",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee3() {
  var actual;
  return regeneratorRuntime.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.next = 2;
          return call({}, interpreters, basicMultiArg, "foo", "bar");

        case 2:
          actual = _context3.sent;
          expect(actual).toEqual("foobar");

        case 4:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee3, this);
})));
test("should be able to process commands in multiple steps",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee4() {
  var actual, expected;
  return regeneratorRuntime.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.next = 2;
          return call({}, interpreters, basicMultistep, "foo");

        case 2:
          actual = _context4.sent;
          expected = {
            s1: "foo1",
            s2: "foo2"
          };
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4, this);
})));
test("should be able to process commands in parallel",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee5() {
  var actual, expected;
  return regeneratorRuntime.wrap(function _callee5$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          _context5.next = 2;
          return call({}, interpreters, basicParallel, "foo");

        case 2:
          actual = _context5.sent;
          expected = {
            s1: "foo1",
            s2: "foo2"
          };
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context5.stop();
      }
    }
  }, _callee5, this);
})));
test("should be able to process commands in parallel, in multiple steps",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee6() {
  var actual, expected;
  return regeneratorRuntime.wrap(function _callee6$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          _context6.next = 2;
          return call({}, interpreters, basicMultistepParallel, "foo");

        case 2:
          actual = _context6.sent;
          expected = {
            s1: "foo1",
            s2: "foo2",
            s3: "foo3",
            s4: "foo4"
          };
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee6, this);
})));
test("should be able to process an empty list of commands",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee7() {
  var expected, actual;
  return regeneratorRuntime.wrap(function _callee7$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          expected = [];
          _context7.next = 3;
          return call({}, interpreters, basicEmpty);

        case 3:
          actual = _context7.sent;
          expect(actual).toEqual(expected);

        case 5:
        case "end":
          return _context7.stop();
      }
    }
  }, _callee7, this);
})));
test("interpreters should be able to return non-promise values",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee8() {
  var _marked, valueReturningInterpreter, actual, expected;

  return regeneratorRuntime.wrap(function _callee8$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          valueReturningInterpreter = function _ref9(value) {
            return regeneratorRuntime.wrap(function valueReturningInterpreter$(_context8) {
              while (1) {
                switch (_context8.prev = _context8.next) {
                  case 0:
                    _context8.next = 2;
                    return cmds.echo(value);

                  case 2:
                    return _context8.abrupt("return", _context8.sent);

                  case 3:
                  case "end":
                    return _context8.stop();
                }
              }
            }, _marked, this);
          };

          _marked =
          /*#__PURE__*/
          regeneratorRuntime.mark(valueReturningInterpreter);
          _context9.next = 4;
          return call({}, interpreters, valueReturningInterpreter, "bar");

        case 4:
          actual = _context9.sent;
          expected = "bar";
          expect(actual).toEqual(expected);

        case 7:
        case "end":
          return _context9.stop();
      }
    }
  }, _callee8, this);
})));
test("interpreters should be able to return promises",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee9() {
  var _marked2, promiseReturningInterpreter, actual, expected;

  return regeneratorRuntime.wrap(function _callee9$(_context11) {
    while (1) {
      switch (_context11.prev = _context11.next) {
        case 0:
          promiseReturningInterpreter = function _ref11(value) {
            return regeneratorRuntime.wrap(function promiseReturningInterpreter$(_context10) {
              while (1) {
                switch (_context10.prev = _context10.next) {
                  case 0:
                    _context10.next = 2;
                    return cmds.echoPromise(value);

                  case 2:
                    return _context10.abrupt("return", _context10.sent);

                  case 3:
                  case "end":
                    return _context10.stop();
                }
              }
            }, _marked2, this);
          };

          _marked2 =
          /*#__PURE__*/
          regeneratorRuntime.mark(promiseReturningInterpreter);
          _context11.next = 4;
          return call({}, interpreters, promiseReturningInterpreter, "bar");

        case 4:
          actual = _context11.sent;
          expected = "bar";
          expect(actual).toEqual(expected);

        case 7:
        case "end":
          return _context11.stop();
      }
    }
  }, _callee9, this);
})));
test("interpreters should be able to return an array of results",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee10() {
  var fn, actual, expected;
  return regeneratorRuntime.wrap(function _callee10$(_context13) {
    while (1) {
      switch (_context13.prev = _context13.next) {
        case 0:
          fn =
          /*#__PURE__*/
          regeneratorRuntime.mark(function fn(a, b) {
            var result;
            return regeneratorRuntime.wrap(function fn$(_context12) {
              while (1) {
                switch (_context12.prev = _context12.next) {
                  case 0:
                    _context12.next = 2;
                    return [cmds.echo(a), cmds.echo(b)];

                  case 2:
                    result = _context12.sent;
                    return _context12.abrupt("return", result);

                  case 4:
                  case "end":
                    return _context12.stop();
                }
              }
            }, fn, this);
          });
          _context13.next = 3;
          return call({}, interpreters, fn, ["foo", "bar"], ["foo", "baz"]);

        case 3:
          actual = _context13.sent;
          expected = [["foo", "bar"], ["foo", "baz"]];
          expect(actual).toEqual(expected);

        case 6:
        case "end":
          return _context13.stop();
      }
    }
  }, _callee10, this);
})));
//# sourceMappingURL=scenarios.spec.js.map