"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var _marked =
/*#__PURE__*/
regeneratorRuntime.mark(basic),
    _marked2 =
/*#__PURE__*/
regeneratorRuntime.mark(basicMultistep);

var _require = require("../core"),
    call = _require.call,
    buildFunctions = _require.buildFunctions;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    functions = _require2.functions,
    cmds = _require2.cmds;

var usesThrowingInterpreter = functions.usesThrowingInterpreter;

var _require3 = require("./test-util"),
    sleep = _require3.sleep;

var coreInterpreters = require("../interpreters");

var coreCmds = require("../cmds");

function basic(message) {
  return regeneratorRuntime.wrap(function basic$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return cmds.echo(message);

        case 2:
          return _context.abrupt("return", _context.sent);

        case 3:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function basicMultistep(message) {
  var s1, s2;
  return regeneratorRuntime.wrap(function basicMultistep$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return cmds.echo(message + "1");

        case 2:
          s1 = _context2.sent;
          _context2.next = 5;
          return cmds.echo(message + "2");

        case 5:
          s2 = _context2.sent;
          return _context2.abrupt("return", {
            s1: s1,
            s2: s2
          });

        case 7:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

test("should add a stack to the context and push the current frame",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee() {
  var telemetry, onCommand, context, now;
  return regeneratorRuntime.wrap(function _callee$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          onCommand = function onCommand(t) {
            telemetry = t;
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context3.next = 5;
          return call(context, interpreters, basic, "foo");

        case 5:
          _context3.next = 7;
          return sleep(10);

        case 7:
          expect(telemetry.context.stack[0].fn).toEqual(basic);
          expect(telemetry.context.stack[0].interpreters).toEqual(interpreters);
          expect(telemetry.context.stack[0].args).toEqual(["foo"]);
          expect(telemetry.context.stack[0].context.onCommand).toEqual(onCommand);
          expect(telemetry.context.stack[0].context.name).toEqual("telemetry");
          expect(telemetry.context.stack[0].context.stack).toEqual(undefined); // Should be serializable

          JSON.stringify(telemetry);

        case 14:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee, this);
})));
test("should add a stack to the context for child calls",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee2() {
  var telemetry, onCommand, context, now;
  return regeneratorRuntime.wrap(function _callee2$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          onCommand = function onCommand(t) {
            telemetry = t;
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context4.next = 5;
          return call(context, interpreters, basic, "foo");

        case 5:
          _context4.next = 7;
          return sleep(10);

        case 7:
          expect(telemetry.context.stack[0].fn).toEqual(basic);
          expect(telemetry.context.stack[0].interpreters).toEqual(interpreters);
          expect(telemetry.context.stack[0].args).toEqual(["foo"]);
          expect(telemetry.context.stack[0].context.onCommand).toEqual(onCommand);
          expect(telemetry.context.stack[0].context.name).toEqual("telemetry");
          expect(telemetry.context.stack[0].context.stack).toEqual(undefined); // Should be serializable

          JSON.stringify(telemetry);

        case 14:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee2, this);
})));
test("onCommand",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee3() {
  var telemetry, onCommand, context, now;
  return regeneratorRuntime.wrap(function _callee3$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          telemetry = [];

          onCommand = function onCommand(t) {
            telemetry.push(t);
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context5.next = 6;
          return call(context, interpreters, basicMultistep, "foo");

        case 6:
          _context5.next = 8;
          return sleep(10);

        case 8:
          expect(telemetry.length).toEqual(2);
          telemetry.forEach(function (t, i) {
            var message = "foo" + (i + 1);
            expect(t.command).toEqual(cmds.echo(message));
            expect(t.fn).toEqual(basicMultistep);
            expect(t.start).toBeGreaterThanOrEqual(now);
            expect(t.index).toEqual(0);
            expect(t.step).toEqual(i);
            expect(t.context.name).toEqual("telemetry");
            expect(t.context.stack[0].fn).toEqual(basicMultistep);
          }); // Should be serializable

          JSON.stringify(telemetry);

        case 11:
        case "end":
          return _context5.stop();
      }
    }
  }, _callee3, this);
})));
test("onCommandComplete",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee4() {
  var telemetry, onCommandComplete, context, now;
  return regeneratorRuntime.wrap(function _callee4$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          telemetry = [];

          onCommandComplete = function onCommandComplete(t) {
            telemetry.push(t);
          };

          context = {
            onCommandComplete: onCommandComplete,
            name: "telemetry"
          };
          now = Date.now();
          _context6.next = 6;
          return call(context, interpreters, basicMultistep, "foo");

        case 6:
          _context6.next = 8;
          return sleep(10);

        case 8:
          expect(telemetry.length).toEqual(2);
          telemetry.forEach(function (t, i) {
            var message = "foo" + (i + 1);
            expect(t.success).toEqual(true);
            expect(t.command).toEqual(cmds.echo(message));
            expect(t.latency).toBeLessThan(5);
            expect(t.start).toBeGreaterThanOrEqual(now);
            expect(t.end).toBeGreaterThanOrEqual(t.start);
            expect(t.index).toEqual(0);
            expect(t.step).toEqual(i);
            expect(t.result).toEqual(message);
            expect(t.context.name).toEqual("telemetry");
            expect(t.context.stack[0].fn).toEqual(basicMultistep);
            expect(t.fn).toEqual(basicMultistep);
          }); // Should be serializable

          JSON.stringify(telemetry);

        case 11:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee4, this);
})));
test("telemetry on error - onCommandComplete",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee5() {
  var telemetry, onCommandComplete, context, now, message;
  return regeneratorRuntime.wrap(function _callee5$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          onCommandComplete = function onCommandComplete(t) {
            telemetry = t;
          };

          context = {
            onCommandComplete: onCommandComplete,
            name: "usesThrowingInterpreter"
          };
          now = Date.now();
          message = "oops";
          _context7.prev = 4;
          _context7.next = 7;
          return call(context, interpreters, usesThrowingInterpreter, message);

        case 7:
          _context7.next = 11;
          break;

        case 9:
          _context7.prev = 9;
          _context7.t0 = _context7["catch"](4);

        case 11:
          _context7.next = 13;
          return sleep(10);

        case 13:
          expect(telemetry.success).toEqual(false);
          expect(telemetry.command).toEqual(cmds.die(message));
          expect(telemetry.latency).toBeLessThan(5);
          expect(telemetry.start).toBeGreaterThanOrEqual(now);
          expect(telemetry.end).toBeGreaterThanOrEqual(telemetry.start);
          expect(telemetry.index).toEqual(0);
          expect(telemetry.step).toEqual(0);
          expect(telemetry.result.message).toEqual("oops");
          expect(telemetry.context.name).toEqual("usesThrowingInterpreter");
          expect(telemetry.context.stack[0].fn).toEqual(usesThrowingInterpreter);
          expect(telemetry.fn).toEqual(usesThrowingInterpreter); // Should be serializable

          JSON.stringify(telemetry);

        case 25:
        case "end":
          return _context7.stop();
      }
    }
  }, _callee5, this, [[4, 9]]);
})));
test("onCall", function (done) {
  var onCall = function onCall(called) {
    expect(called.args).toEqual(["foo", "bar", "baz"]);
    done();
  };

  var context = {
    onCall: onCall
  };
  call(context, interpreters, basicMultistep, "foo", "bar", "baz");
});
test("onCallComplete", function (done) {
  var now = Date.now();

  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(true);
    expect(complete.fn).toEqual(basic);
    expect(complete.result).toEqual("foo");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };

  var context = {
    onCallComplete: onCallComplete,
    name: "telemetry"
  };
  call(context, interpreters, basic, "foo");
});
test("onCallComplete for errors from interpreters", function (done) {
  var now = Date.now();

  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(usesThrowingInterpreter);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };

  var context = {
    onCallComplete: onCallComplete,
    name: "telemetry"
  };
  call(context, interpreters, usesThrowingInterpreter, "foo").catch(function (e) {
    return e;
  });
});
test("onCallComplete for errors from function body", function (done) {
  var _marked3 =
  /*#__PURE__*/
  regeneratorRuntime.mark(throwFromBody);

  var now = Date.now();

  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(throwFromBody);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };

  function throwFromBody() {
    return regeneratorRuntime.wrap(function throwFromBody$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context8.stop();
        }
      }
    }, _marked3, this);
  }

  var context = {
    onCallComplete: onCallComplete,
    name: "telemetry"
  };
  call(context, interpreters, throwFromBody).catch(function (e) {
    return e;
  });
});
test("onCallComplete for errors from function body when using buildFunctions", function (done) {
  var _marked4 =
  /*#__PURE__*/
  regeneratorRuntime.mark(throwFromBody);

  var now = Date.now();

  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(throwFromBody);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };

  function throwFromBody() {
    return regeneratorRuntime.wrap(function throwFromBody$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return cmds.echo("foo");

          case 2:
            throw new Error("oops");

          case 3:
          case "end":
            return _context9.stop();
        }
      }
    }, _marked4, this);
  }

  var context = {
    onCallComplete: onCallComplete,
    name: "telemetry"
  };
  var built = buildFunctions(context, interpreters, {
    throwFromBody: throwFromBody
  });
  built.throwFromBody().catch(function (e) {
    return e;
  });
});
test("telemetry onError - interpreter",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee6() {
  var error, errorContext, callCount, onError, context, now, message;
  return regeneratorRuntime.wrap(function _callee6$(_context10) {
    while (1) {
      switch (_context10.prev = _context10.next) {
        case 0:
          callCount = 0;

          onError = function onError(e, c) {
            error = e;
            errorContext = c;
            callCount++;
          };

          context = {
            onError: onError,
            name: "usesThrowingInterpreter"
          };
          now = Date.now();
          message = "oops";
          _context10.prev = 5;
          _context10.next = 8;
          return call(context, interpreters, usesThrowingInterpreter, message);

        case 8:
          _context10.next = 12;
          break;

        case 10:
          _context10.prev = 10;
          _context10.t0 = _context10["catch"](5);

        case 12:
          _context10.next = 14;
          return sleep(10);

        case 14:
          expect(error.message).toEqual("oops");
          expect(errorContext.name).toEqual("usesThrowingInterpreter");
          expect(errorContext.stack[0].fn).toEqual(usesThrowingInterpreter);
          expect(errorContext.stack[0].args).toEqual([message]);
          expect(callCount).toEqual(1); // Should be serializable

          JSON.stringify(error);

        case 20:
        case "end":
          return _context10.stop();
      }
    }
  }, _callee6, this, [[5, 10]]);
})));
test("telemetry onError - function body",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee7() {
  var _marked5, error, errorContext, callCount, onError, context, now, throwFromBody;

  return regeneratorRuntime.wrap(function _callee7$(_context12) {
    while (1) {
      switch (_context12.prev = _context12.next) {
        case 0:
          throwFromBody = function _ref8() {
            return regeneratorRuntime.wrap(function throwFromBody$(_context11) {
              while (1) {
                switch (_context11.prev = _context11.next) {
                  case 0:
                    _context11.next = 2;
                    return cmds.echo("foo");

                  case 2:
                    throw new Error("oops");

                  case 3:
                  case "end":
                    return _context11.stop();
                }
              }
            }, _marked5, this);
          };

          _marked5 =
          /*#__PURE__*/
          regeneratorRuntime.mark(throwFromBody);
          callCount = 0;

          onError = function onError(e, c) {
            error = e;
            errorContext = c;
            callCount++;
          };

          context = {
            onError: onError,
            name: "throwFromBody"
          };
          now = Date.now();
          _context12.prev = 6;
          _context12.next = 9;
          return call(context, interpreters, throwFromBody, "arg1", "arg2");

        case 9:
          _context12.next = 13;
          break;

        case 11:
          _context12.prev = 11;
          _context12.t0 = _context12["catch"](6);

        case 13:
          _context12.next = 15;
          return sleep(10);

        case 15:
          expect(error.message).toEqual("oops");
          expect(errorContext.name).toEqual("throwFromBody");
          expect(errorContext.stack[0].fn).toEqual(throwFromBody);
          expect(errorContext.stack[0].args).toEqual(["arg1", "arg2"]);
          expect(callCount).toEqual(1); // Should be serializable

          JSON.stringify(error);

        case 21:
        case "end":
          return _context12.stop();
      }
    }
  }, _callee7, this, [[6, 11]]);
})));
test("should not mutate stack",
/*#__PURE__*/
_asyncToGenerator(
/*#__PURE__*/
regeneratorRuntime.mark(function _callee8() {
  var _marked6, stack1, stack2, onCommand, nestedCall, context, now;

  return regeneratorRuntime.wrap(function _callee8$(_context14) {
    while (1) {
      switch (_context14.prev = _context14.next) {
        case 0:
          nestedCall = function _ref10() {
            return regeneratorRuntime.wrap(function nestedCall$(_context13) {
              while (1) {
                switch (_context13.prev = _context13.next) {
                  case 0:
                    if (!stack2) {
                      _context13.next = 2;
                      break;
                    }

                    return _context13.abrupt("return");

                  case 2:
                    _context13.next = 4;
                    return coreCmds.setImmediate(nestedCall);

                  case 4:
                  case "end":
                    return _context13.stop();
                }
              }
            }, _marked6, this);
          };

          _marked6 =
          /*#__PURE__*/
          regeneratorRuntime.mark(nestedCall);

          onCommand = function onCommand(t) {
            if (!stack1) {
              stack1 = t.context.stack;
              return;
            }

            if (!stack2) {
              stack2 = t.context.stack;
              return;
            }
          };

          context = {
            onCommand: onCommand,
            name: "telemetry"
          };
          now = Date.now();
          _context14.next = 7;
          return call(context, coreInterpreters, nestedCall, "foo");

        case 7:
          _context14.next = 9;
          return sleep(10);

        case 9:
          expect(stack1).toBeTruthy();
          expect(stack2).toBeTruthy();
          expect(stack1).not.toBe(stack2);

        case 12:
        case "end":
          return _context14.stop();
      }
    }
  }, _callee8, this);
})));
//# sourceMappingURL=telemetry.spec.js.map